"use strict";
// This file is copied from github.com/OfficeDev/office-ui-fabric-react/blob/6.0/packages/utilities/src/autobind.ts
// As the autobind is a breaking change when migrating fabric from 6 to 7 and the autobind has been widely used
// in this repo, the best way to unblock the problem is to implement our own autobind. As a result of that, we are
// copying this file into the repo.
Object.defineProperty(exports, "__esModule", { value: true });
exports.autobind = void 0;
/**
 * Autobind is a utility for binding methods in a class. This simplifies tagging methods as being "bound" to the
 * this pointer so that they can be used in scenarios that simply require a function callback.
 *
 * @internal
 * @deprecated Prefer arrow function expressions or Function.prototype.bind to bind function execution context.
 */
function autobind(
// tslint:disable-next-line:no-any
target, key, descriptor) {
    var fn = descriptor.value;
    var defining = false;
    return {
        configurable: true,
        get: function () {
            if (defining || (fn && this === fn.prototype) || this.hasOwnProperty(key)) {
                return fn;
            }
            // Bind method only once, and update the property to return the bound value from now on
            var fnBound = fn && fn.bind(this);
            defining = true;
            Object.defineProperty(this, key, {
                configurable: true,
                writable: true,
                enumerable: true,
                value: fnBound
            });
            defining = false;
            return fnBound;
        },
        // tslint:disable-next-line:no-any
        set: function (newValue) {
            Object.defineProperty(this, key, {
                configurable: true,
                writable: true,
                enumerable: true,
                value: newValue
            });
        }
    };
}
exports.autobind = autobind;
//# sourceMappingURL=autobind.js.map